/*
 * Copyright  2008 Nokia Corporation.
 */

#include "cdrawingengine.h"
#include "ctoolbarcontrol.h"
#include <BITSTD.H>

CDrawingEngine::CDrawingEngine (RWindow& aWindow, CWindowGc& aGc, CFbsBitGc* aBgc, CToolbarControl* aToolbarControl)
: iWindow(aWindow), iGc(aGc),iBgc(aBgc), iToolbarControl(aToolbarControl)
    {
    }

CDrawingEngine::~CDrawingEngine ( )
    {
    }

CDrawingEngine* CDrawingEngine::NewLC (RWindow& aWindow, CWindowGc& aGc, CFbsBitGc* aBgc, CToolbarControl* aToolbarControl)
    {
    CDrawingEngine* self = new (ELeave)CDrawingEngine(aWindow,aGc,aBgc,aToolbarControl);
    CleanupStack::PushL (self );
    self->ConstructL ( );
    return self;
    }

CDrawingEngine* CDrawingEngine::NewL (RWindow& aWindow, CWindowGc& aGc, CFbsBitGc* aBgc, CToolbarControl* aToolbarControl)
    {
    CDrawingEngine* self=CDrawingEngine::NewLC (aWindow,aGc,aBgc,aToolbarControl);
    CleanupStack::Pop ( ); // self;
    return self;
    }

void CDrawingEngine::ConstructL ( )
    {
    ResetDrawing();
    }

void CDrawingEngine::SetNewBgc(CFbsBitGc* aBgc)
    {
    // Set new backbuffer context for drawing
    iBgc = aBgc;
    
    SetPenSize(iSize);
    SetColor(iColor);
    }

void CDrawingEngine::SetUseBgc(TBool aUseBgc)
    {
    // CFbsBitGc or CWindowGc in use
    iUseBgc = aUseBgc;
    }

void CDrawingEngine::ResetDrawing()
    {
    iLinePoint1 = TPoint(-1,-1);
    iLinePoint2 = TPoint(-1,-1);
    iCircleCenter = TPoint(-1,-1);
    }

void CDrawingEngine::DoDrawing(const TPointerEvent& aPointerEvent)
    {
    switch (iToolbarControl->ActiveTool())
        {
        case ENoTool:
            {
            break;
            }
        case EPencil:
            {
            DoFreeDraw(aPointerEvent);
            break;
            }
        case ELine:
            {
            DoLine(aPointerEvent);
            break;
            }
        case ECircle:
            {
            DoCircle(aPointerEvent);
            break;
            }
        case ERect:
            {
            DoRect(aPointerEvent);
            break;
            }
        default:
            {
            break;
            }
        }
    }

void CDrawingEngine::SetPenSize(TSize aSize)
    {
    iSize = aSize;
    if (iUseBgc)
        {
        iBgc->SetPenSize(iSize);
        }
    else
        {
        iGc.SetPenSize(iSize);
        }
    }

void CDrawingEngine::SetColor(TRgb aColor)
    {
    iColor = aColor;
    if (iUseBgc)
        {
        iBgc->SetPenColor(iColor);
        }
    else
        {
        iGc.SetPenColor(iColor);
        }
    }

TBool CDrawingEngine::CircleReady()
    {
    if (iCircleCenter.iX > -1 && iCircleCenter.iY > -1)
        {
        return ETrue;
        }
    else
        {
        return EFalse;
        }
    }

TBool CDrawingEngine::RectangleReady()
    {
    if (iCircleCenter.iX > -1 && iCircleCenter.iY > -1)
        {
        return ETrue;
        }
    else
        {
        return EFalse;
        }
    }

TBool CDrawingEngine::LineReady()
    {
    if (iLinePoint1.iX > -1 && iLinePoint2.iX > -1)
        {
        return ETrue;
        }
    else
        {
        return EFalse;
        }
    }


void CDrawingEngine::DoLine(const TPointerEvent& aPointerEvent)
    {
    if (aPointerEvent.iType == TPointerEvent::EButton1Down)
        {
        ResetDrawing();
        iLinePoint1 = aPointerEvent.iPosition;
        }
    else if(aPointerEvent.iType == TPointerEvent::EDrag)
        {
        iLinePoint2 = aPointerEvent.iPosition;
        if (LineReady())
            {
            if (iUseBgc)
                {
                iBgc->DrawLine(iLinePoint1, iLinePoint2);
                }
            else
                {
                iGc.DrawLine(iLinePoint1, iLinePoint2);
                }
            }
        }
    else if(aPointerEvent.iType == TPointerEvent::EButton1Up)
        {
        iLinePoint2 = aPointerEvent.iPosition;
        if (LineReady())
            {
            if (iUseBgc)
                {
                iBgc->DrawLine(iLinePoint1, iLinePoint2);
                }
            else
                {
                iGc.DrawLine(iLinePoint1, iLinePoint2);
                }
            }
        ResetDrawing();
        }
    }

void CDrawingEngine::DoFreeDraw(const TPointerEvent& aPointerEvent)
    {
    if (aPointerEvent.iType == TPointerEvent::EButton1Down)
        {
        ResetDrawing();
        iLinePoint1 = aPointerEvent.iPosition;
        }
    else if(aPointerEvent.iType == TPointerEvent::EDrag)
        {
        iLinePoint2 = aPointerEvent.iPosition;
        if (LineReady())
            {
            if (iUseBgc)
                {
                iBgc->DrawLine(iLinePoint1, iLinePoint2);
                }
            else
                {
                iGc.DrawLine(iLinePoint1, iLinePoint2);
                }
            iLinePoint1 = iLinePoint2;
            }
        }
    else if(aPointerEvent.iType == TPointerEvent::EButton1Up)
        {
        iLinePoint2 = aPointerEvent.iPosition;
        if (LineReady())
            {
            if (iUseBgc)
                iBgc->DrawLine(iLinePoint1, iLinePoint2);
            else
                {
                iGc.DrawLine(iLinePoint1, iLinePoint2);
                }
            }
        ResetDrawing();
        }
    }

void CDrawingEngine::DoCircle(const TPointerEvent& aPointerEvent)
    {
    if (aPointerEvent.iType == TPointerEvent::EButton1Down)
        {
        ResetDrawing();
        iCircleCenter = aPointerEvent.iPosition;
        }
    else if(aPointerEvent.iType == TPointerEvent::EDrag)
        {
        TInt width = iCircleCenter.iX - aPointerEvent.iPosition.iX;
        TInt height = iCircleCenter.iY - aPointerEvent.iPosition.iY;
        TRect newRect = TRect(iCircleCenter, TSize(width,height));

        newRect.Move(-width,-height);
        newRect.Normalize();
        
        if (CircleReady())
            {
            if (iUseBgc)
                iBgc->DrawEllipse(newRect);
            else
                iGc.DrawEllipse(newRect);
            }
        
        }
    else if(aPointerEvent.iType == TPointerEvent::EButton1Up)
        {
        TInt width = iCircleCenter.iX - aPointerEvent.iPosition.iX;
        TInt height = iCircleCenter.iY - aPointerEvent.iPosition.iY;
        TRect newRect = TRect(iCircleCenter, TSize(width,height));

        newRect.Move(-width,-height);
        newRect.Normalize();
        
        if (CircleReady())
            {
            if (iUseBgc)
                iBgc->DrawEllipse(newRect);
            else
                iGc.DrawEllipse(newRect);
            }
        ResetDrawing();
        }
    }

void CDrawingEngine::DoRect(const TPointerEvent& aPointerEvent)
    {
    if (aPointerEvent.iType == TPointerEvent::EButton1Down)
        {
        ResetDrawing();
        iCircleCenter = aPointerEvent.iPosition;
        }
    else if(aPointerEvent.iType == TPointerEvent::EDrag)
        {
        TInt width = iCircleCenter.iX - aPointerEvent.iPosition.iX;
        TInt height = iCircleCenter.iY - aPointerEvent.iPosition.iY;

        TRect newRect = TRect(iCircleCenter, TSize(width,height));
        newRect.Move(-width,-height);
        newRect.Normalize();
        
        if (RectangleReady())
            {
            if (iUseBgc)
                iBgc->DrawRect(newRect);
            else
                iGc.DrawRect(newRect);
            }
        }
    else if(aPointerEvent.iType == TPointerEvent::EButton1Up)
        {
        TInt width = iCircleCenter.iX - aPointerEvent.iPosition.iX;
        TInt height = iCircleCenter.iY - aPointerEvent.iPosition.iY;

        TRect newRect = TRect(iCircleCenter, TSize(width,height));
        newRect.Move(-width,-height);
        newRect.Normalize();
        
        if (RectangleReady())
            {
            if (iUseBgc)
                iBgc->DrawRect(newRect);
            else
                iGc.DrawRect(newRect);
            }
        ResetDrawing();
        }
    }

